﻿using System;
using System.Diagnostics;
using System.Threading.Tasks;
using com.orbitgis.tool.message;
using orbitgt.mapping3d.sdk.viewer;

namespace plugin_3dm_arcgispro
{
    /// <summary>
    /// Class UpdateGlobal manages the updates to the global layer
    /// </summary>
    class UpdateGlobal
    {
        private MapObjectLayer layer;
        private PluginArcGIS viewerHost;
        private long updateTime;
        private long lastRequestTime;
        private bool isUpdating;
        private AList<MapObject> mapObjects;

        /// <summary>
        /// Create a new updater
        /// </summary>
        internal UpdateGlobal(MapObjectLayer layer, PluginArcGIS viewerHost)
        {
            this.layer = layer;
            this.viewerHost = viewerHost;
            this.updateTime = 0;
            this.lastRequestTime = 0;
            this.isUpdating = false;
            this.mapObjects = null;
        }

        /// <summary>
        /// Called when the globals have been changed and the display need to be updated
        /// </summary>
        internal void updatedGlobals()
        {
            this.updateTime = DateTime.Now.Ticks;
        }

        /// <summary>
        /// Check for updates
        /// </summary>
        internal void checkForUpdates()
        {
            // already updating?
            if (this.isUpdating) return;
            // already up-to-date?
            if (this.lastRequestTime > this.updateTime) return;
            long currentTime = DateTime.Now.Ticks;
            // throttle the rate of update requests
            if ((this.lastRequestTime > 0) && (currentTime < this.lastRequestTime + 100 * TimeSpan.TicksPerMillisecond)) return;
            // throttle the rate of view changes (fast panning)
            if ((this.updateTime > 0) && (currentTime < this.updateTime + 100 * TimeSpan.TicksPerMillisecond)) return;
            // start the update task (to free the calling timer thread)
            this.isUpdating = true;
            this.lastRequestTime = currentTime;
            Task.Run(() => this.update());
        }

        /// <summary>
        /// Update
        /// </summary>
        private void update()
        {
            try
            {
                Message.WriteLine("Requesting globals");
                this.mapObjects = this.viewerHost.getViewer().getGlobals();
                if (this.mapObjects != null) Message.WriteLine("Got " + this.mapObjects.size() + " mapobjects");
                this.isUpdating = false;
                /* Update the layer */
                this.layer.update(this.mapObjects);
                this.viewerHost.RequestMapUpdate("updated globals");
                return;
            }
            catch (Exception e)
            {
                /* Log the exception */
                Message.WriteLine("Error while updating globals: " + e);
                this.isUpdating = false;
                return;
            }
        }
    }
}
